#ifndef bqtRockEditROmHH
#define bqtRockEditROmHH

#include <vector>
#include <string>

#include "romaddr.hh"
#include "space.hh"

extern std::string InputFN;
extern std::string OutputFN;

extern class ROM
{
public:
    ROM();
    ~ROM();
    
    /* ROM handling */
    
    void ReRead();
    void Resize(unsigned npages);
    
    /* Free space */
    
    void AddFreeSpace(unsigned addr, unsigned len)
    {
        space.Add(NES2Space(addr), len);
    }
    void DelFreeSpace(unsigned addr, unsigned len)
    {
        space.Del(NES2Space(addr), len);
    }
    unsigned FindFreeSpaceIn(unsigned bank, unsigned length)
    {
        return Space2NES((bank << 16) | space.Find(bank, length));
    }
    
    
    void SpaceReport()
    {
        space.Report();
    }
    
    /* Reading ROM */
    
    unsigned char Read(unsigned addr) const
    {
        return data[NES2ROMaddr(addr)];
    }
    unsigned short ReadWord(unsigned addr) const
    {
        addr = NES2ROMaddr(addr);
        return data[addr] | (data[addr+1]<<8);
    }
    unsigned ReadPointer(unsigned addr) const
    {
        unsigned rawptr = ReadWord(addr);
        
        if(rawptr >= 0xC000) return NES2ROMaddr(rawptr);
        if(addr / 0x4000 == 3)
        {
            // argh, have to guess the page.
            // don't guess.
        }
        return (addr & 0xFF0000) | rawptr;
    }
    void ReadData(unsigned addr, unsigned n, unsigned char* target) const
    {
        addr = NES2ROMaddr(addr);
        for(unsigned a=0; a<n; ++a) target[a] = data[addr+a];
    }
    
    /* Writing ROM */

    void Write(unsigned addr, unsigned char val)
    {
        data[NES2ROMaddr(addr)] = val;
    }

    void WriteBytes(unsigned addr, unsigned n, const unsigned char* source)
    {
        addr = NES2ROMaddr(addr);
        for(unsigned a=0; a<n; ++a) data[addr+a] = source[a];
    }
    
    
    void WritePointer(unsigned ptroffs, unsigned offs)
    {
        unsigned addr = NES2ROMaddr(ptroffs);
        data[addr+0] = offs&0xFF;
        data[addr+1] = offs>>8;
    }
    
private:
    unsigned ROM2Space(unsigned addr) const
    {
        return NES2Space(ROM2NESaddr(addr));
    }
    unsigned NES2Space(unsigned addr) const
    {
        unsigned char bank;
        unsigned offs;
        SplitNESaddr(addr, bank,offs);
        unsigned result = (bank<<16) | offs;
        //std::fprintf(stderr, "NES2Space(0x%X) = %X (%X:%X)\n",addr,result,bank,offs);
        return result;
    }
    unsigned Space2NES(unsigned addr) const
    {
        return MakeNESaddr(addr>>16, addr&0xFFFF);
    }
    
private:
    std::vector<unsigned char> data;
    freespacemap space;

private:
    ROM(const ROM&);
    void operator =(const ROM&);
} ROM;

#endif
